package bll

import (
	"DBMS/dal"
	"DBMS/dbmodel"
	"github.com/google/uuid"
	"github.com/robfig/cron/v3"
	"go.mongodb.org/mongo-driver/bson/primitive"
	"log"
	"os"
	"strconv"
	"time"
)

var DailyStatisticsInfo dbmodel.DailyStatisticsMetaInfoV1

type OnlineUserInfo struct {
	UserInfo          dbmodel.UserMetaInfoV1
	Token             string
	expired           bool
	LastHeartBeatTime time.Time
}

var OnlineUserInfoCache = map[string]OnlineUserInfo{}

func CronAutoSaveDailyStatistics() {
	c := cron.New(cron.WithSeconds(), cron.WithChain(cron.SkipIfStillRunning(cron.DefaultLogger)), cron.WithLogger(
		cron.VerbosePrintfLogger(log.New(os.Stdout, "cron: ", log.LstdFlags))))
	EntryID, err := c.AddFunc("@daily", func() {
		log.Println(time.Now(), "CronAutoSaveDailyStatistics...")
		timePoint := time.Now()
		year, mouth, day := timePoint.Date()
		DailyStatisticsInfo.Base.Id = primitive.NewObjectID()
		DailyStatisticsInfo.Base.Uuid = uuid.NewString()
		DailyStatisticsInfo.Base.DataAccessModelVersion = "V1"
		DailyStatisticsInfo.Name = strconv.Itoa(year) + "-" + mouth.String() + "-" + strconv.Itoa(day)
		DailyStatisticsInfo.Day = strconv.Itoa(year) + "-" + mouth.String() + "-" + strconv.Itoa(day)
		DailyStatisticsInfo.Description = "Auto Generated by CronAutoSaveDailyStatistics."
		log.Println("TimerEvent: Time - " + timePoint.String() + " , Event - CronAutoSaveDailyStatistics performed.")

		dal.CreateDailyStatistics(DailyStatisticsInfo, dal.GetDbInstance())
	})
	log.Println(time.Now(), EntryID, err)
	c.Start()
}

func CronHeartBeatValidationAndRefresh() {
	c := cron.New(cron.WithSeconds(), cron.WithChain(cron.SkipIfStillRunning(cron.DefaultLogger)), cron.WithLogger(
		cron.VerbosePrintfLogger(log.New(os.Stdout, "cron: ", log.LstdFlags))))
	EntryID, err := c.AddFunc("*/60 * * * * *", func() {
		log.Println(time.Now(), "CronHeartBeatValidationAndRefresh...")
		for key, onlineUserInfo := range OnlineUserInfoCache {
			if time.Now().After(onlineUserInfo.LastHeartBeatTime) || onlineUserInfo.expired {
				onlineUserInfo.expired = true
				delete(OnlineUserInfoCache, key)
				log.Println("User " + onlineUserInfo.UserInfo.Name + " HeartBeat expired")
			}
		}
	})
	log.Println(time.Now(), EntryID, err)
	c.Start()
}

//func TestCron() {
//	c := cron.New(cron.WithSeconds(), cron.WithChain(cron.SkipIfStillRunning(cron.DefaultLogger)), cron.WithLogger(
//		cron.VerbosePrintfLogger(log.New(os.Stdout, "cron: ", log.LstdFlags))))
//	i := 1
//	EntryID, err := c.AddFunc("*/5 * * * * *", func() {
//		log.Println(time.Now(), "每5s一次:", i)
//		time.Sleep(time.Second * 6)
//		i++
//	})
//	log.Println(time.Now(), EntryID, err)
//
//	c.Start()
//}
